#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
reload(sys)
sys.setdefaultencoding('utf8')
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from ui_mainwindow import Ui_MainWindow
from key_service import KeyService
from display_service import DisplayService
from switchers_service import SwitchersService
import fcntl
import keybinder


class KylinDisplaySwitch(QWidget):

    key_service = None
    display_service = None
    switchers_service = None
    settings = None
    current_button = 0

    def __init__(self, parent = None):
        super(KylinDisplaySwitch, self).__init__(parent)

        self.check_singleton()

        self.init_ui()

        self.display_service = DisplayService()
        self.switchers_service = SwitchersService()
        self.settings = QSettings("kylinos.cn", "KylinDisplaySwitch")

        self.init_wifi_bt()

        self.check_trolltech_conf()

    # 如果当前用户目录中没有Trolltech.conf，则创建时赋权限，让其他用户可以访问，必须配合session-setup-script使用
    def check_trolltech_conf(self):
        if not os.path.exists("%s/.config/" % os.environ['HOME']):
            os.system("mkdir %s/.config/" % os.environ['HOME'])
        if not os.path.exists("%s/.config/Trolltech.conf" % os.environ['HOME']):
            os.system("touch %s/.config/Trolltech.conf" % os.environ['HOME'])
        os.system("chown %s:%s %s/.config/" % (os.environ['USER'], os.environ['USER'], os.environ['HOME']))
        os.system("chown %s:%s %s/.config/Trolltech.conf" % (os.environ['USER'], os.environ['USER'], os.environ['HOME']))
        os.system("chmod 777 %s/.config" % os.environ['HOME'])
        os.system("chmod 777 %s/.config/Trolltech.conf" % os.environ['HOME'])

    # 单例
    def check_singleton(self):
        self.instance_file = open("instance_kds.lock", 'w')
        try:
            fcntl.lockf(self.instance_file, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except IOError:
            print "已有一个实例在运行..."
            sys.exit()

    def init_ui(self):
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.ToolTip)

        self.ui.centralWidget.setAutoFillBackground(True)
        pc = QPalette()
        img_bg = QPixmap("res/bg.png")
        pc.setBrush(QPalette.Window, QBrush(img_bg))
        self.ui.centralWidget.setPalette(pc)

        self.ui.lb_mode_1.setText("仅计算机")
        self.ui.lb_mode_2.setText("复制")
        self.ui.lb_mode_3.setText("扩展")
        self.ui.lb_mode_4.setText("仅投影仪")

        self.ui.bg_mode_1.setStyleSheet("QWidget{background-image:url(res/btn1.png); border:0px;}")
        self.ui.bg_mode_2.setStyleSheet("QWidget{background-image:url(res/btn2.png); border:0px;}")
        self.ui.bg_mode_3.setStyleSheet("QWidget{background-image:url(res/btn3.png); border:0px;}")
        self.ui.bg_mode_4.setStyleSheet("QWidget{background-image:url(res/btn4.png); border:0px;}")

        self.ui.split_1.setStyleSheet("QLabel{background-image:url(res/split.png); border:0px;}")
        self.ui.split_2.setStyleSheet("QLabel{background-image:url(res/split.png); border:0px;}")
        self.ui.split_3.setStyleSheet("QLabel{background-image:url(res/split.png); border:0px;}")

        self.ui.mode_1.setStyleSheet("QPushButton{background-image:url(res/hover.png); border:0px;}")
        self.ui.mode_2.setStyleSheet("QPushButton{background-image:url(res/hover.png); border:0px;}")
        self.ui.mode_3.setStyleSheet("QPushButton{background-image:url(res/hover.png); border:0px;}")
        self.ui.mode_4.setStyleSheet("QPushButton{background-image:url(res/hover.png); border:0px;}")

        self.ui.f5Widget.setStyleSheet("QWidget{background-image:url(res/bg_f5.png); border:0px;}")
        self.ui.btnHide.setStyleSheet("QPushButton{background-image:url(res/__Null__.png); border:0px;}")

        self.ui.mode_1.setFocusPolicy(Qt.NoFocus)
        self.ui.mode_2.setFocusPolicy(Qt.NoFocus)
        self.ui.mode_3.setFocusPolicy(Qt.NoFocus)
        self.ui.mode_4.setFocusPolicy(Qt.NoFocus)

        self.ui.btnWifi.setFocusPolicy(Qt.NoFocus)
        self.ui.btnBt.setFocusPolicy(Qt.NoFocus)

        # shadow = QGraphicsDropShadowEffect(self)
        # shadow.setOffset(5, 5)
        # shadow.setColor(Qt.gray)
        # shadow.setBlurRadius(15)
        # self.setGraphicsEffect(shadow)

        self.timer_tip = QTimer()
        self.timer_tip.timeout.connect(self.slot_hide_tip)
        self.ui.tipWidget.hide()

        self.ui.btnWifi.clicked.connect(self.slot_switch_wifi)
        self.ui.btnBt.clicked.connect(self.slot_switch_bt)
        self.ui.btnHide.clicked.connect(self.slot_hide_f5_widget)

    # 启动时将WIFI、蓝牙调整为上次设置的状态
    def init_wifi_bt(self):
        wifi_settings = self.settings.value("is_wifi_on").toBool()
        bt_settings = self.settings.value("is_bt_on").toBool()

        is_wifi_on = self.switchers_service.get_wifi_mode()
        is_bt_on = self.switchers_service.get_bt_mode()

        if(wifi_settings != is_wifi_on):
            self.switchers_service.switch_wifi()

        if(bt_settings != is_bt_on):
            self.switchers_service.switch_bt()

    # 开始监听键盘
    def start_listen(self):
        self.key_service = KeyService(self)
        self.key_service.start_key_threads()

    def switch_button(self):
        self.ui.mode_1.hide()
        self.ui.mode_2.hide()
        self.ui.mode_3.hide()
        self.ui.mode_4.hide()

        self.current_button += 1

        if self.current_button > 4:
            self.current_button = 1

        if self.current_button == 1:
            self.ui.mode_1.show()
        if self.current_button == 2:
            self.ui.mode_2.show()
        if self.current_button == 3:
            self.ui.mode_3.show()
        if self.current_button == 4:
            self.ui.mode_4.show()

    # 图形界面切换模式
    def slot_switch_select(self):
        self.timer_tip.stop()
        self.resize(744, 126)
        self.ui.tipWidget.hide()
        self.ui.f5Widget.hide()
        self.ui.centralWidget.resize(744, 126)
        self.ui.centralWidget.show()
        self.show()

        # 连接多个屏幕时维持程序位置
        desktop = QApplication.desktop()
        if(desktop.screenCount > 1):
            desktop = desktop.screenGeometry(0)
        self.move((desktop.width() - self.width()) / 2, (desktop.height() - self.height()) *10 / 21)

        self.switch_button()

    # 确认切换到当前选择的模式
    def slot_switch_confirm(self):
        self.hide()

        (current_mode, flag) = self.display_service.switch_display(self.current_button)
        # 下次触发时确保当前模式为高亮
        self.current_button = current_mode - 1

    # 显示WIFI和蓝牙开关界面
    def slot_show_f5_widget(self):
        is_wifi_on = self.switchers_service.get_wifi_mode()
        is_bt_on = self.switchers_service.get_bt_mode()
        if(is_wifi_on == True):
            self.ui.btnWifi.setStyleSheet("QPushButton{background-image:url(res/wifi_on.png); border:0px;}")
        else:
            self.ui.btnWifi.setStyleSheet("QPushButton{background-image:url(res/wifi_off.png); border:0px;}")
        if(is_bt_on == True):
            self.ui.btnBt.setStyleSheet("QPushButton{background-image:url(res/bt_on.png); border:0px;}")
        else:
            self.ui.btnBt.setStyleSheet("QPushButton{background-image:url(res/bt_off.png); border:0px;}")

        self.timer_tip.stop()
        self.resize(510, 242)
        desktop = QApplication.desktop()
        if (desktop.screenCount > 1):
            desktop = desktop.screenGeometry(0)
        self.move((desktop.width() - self.width()) / 2, (desktop.height() - self.height()) / 2)
        self.ui.centralWidget.hide()
        self.ui.tipWidget.hide()
        self.ui.f5Widget.show()
        self.show()

    # 开关WIFI
    def slot_switch_wifi(self):
        self.switchers_service.switch_wifi()
        is_wifi_on = self.switchers_service.get_wifi_mode()
        if(is_wifi_on == True):
            self.ui.btnWifi.setStyleSheet("QPushButton{background-image:url(res/wifi_on.png); border:0px;}")
            self.settings.setValue("is_wifi_on", True)
        else:
            self.ui.btnWifi.setStyleSheet("QPushButton{background-image:url(res/wifi_off.png); border:0px;}")
            self.settings.setValue("is_wifi_on", False)

        self.settings.sync()

        self.timer_tip.start(800)

    # 开关蓝牙
    def slot_switch_bt(self):
        self.switchers_service.switch_bt()
        is_bt_on = self.switchers_service.get_bt_mode()
        if(is_bt_on == True):
            self.ui.btnBt.setStyleSheet("QPushButton{background-image:url(res/bt_on.png); border:0px;}")
            self.settings.setValue("is_bt_on", True)
        else:
            self.ui.btnBt.setStyleSheet("QPushButton{background-image:url(res/bt_off.png); border:0px;}")
            self.settings.setValue("is_bt_on", False)

        self.settings.sync()

        self.timer_tip.start(800)

    def slot_hide_f5_widget(self):
        self.timer_tip.stop()
        self.ui.f5Widget.hide()
        self.hide()

    # 开关触摸板提示
    def slot_switch_touchpad(self, is_on):
        self.timer_tip.stop()
        self.timer_tip.start(3000)
        if(is_on == True):
            self.ui.tipWidget.setStyleSheet("QWidget{background-image:url(res/touchpad-on.png); border:0px;}")
        else:
            self.ui.tipWidget.setStyleSheet("QWidget{background-image:url(res/touchpad-off.png); border:0px;}")

        self.resize(190, 190)
        desktop = QApplication.desktop()
        if (desktop.screenCount > 1):
            desktop = desktop.screenGeometry(0)
        self.move((desktop.width() - self.width()) / 2, (desktop.height() - self.height()) / 2)
        self.ui.centralWidget.hide()
        self.ui.f5Widget.hide()
        self.ui.tipWidget.show()
        self.show()

    def slot_hide_tip(self):
        self.timer_tip.stop()
        self.ui.tipWidget.hide()
        self.ui.f5Widget.hide()
        self.hide()


def callback():
    pass


def main():
    app = QApplication(sys.argv)
    QTextCodec.setCodecForTr(QTextCodec.codecForName("UTF-8"))
    QTextCodec.setCodecForCStrings(QTextCodec.codecForName("UTF-8"))

    globalfont = QFont()
    globalfont.setPixelSize(16)
    globalfont.setFamily("方正黑体_GBK")
    app.setFont(globalfont)

    w = KylinDisplaySwitch()

    keystr = "<Super>P"
    keybinder.bind(keystr, callback)

    w.start_listen()

    sys.exit(app.exec_())


if __name__ == '__main__':
    main()